'use strict'
var GOBJ = GOBJ || {};

GOBJ.board = new board();

function board() {
  var self = this;

  self.shuffle = function(o){
      for(var j, x, i = o.length; i; j = Math.floor(Math.random() * i), x = o[--i], o[i] = o[j], o[j] = x);
      return o;
  }

  self.squares = [];
  self.sets = [];
  self.tops = [];

  self.aiShotCount = 0;

  // this is set by caller
  self.dumbness = false;

  self.dumbmod = 3;

  self.hasWinner = false;

  // horizontal rows
  self.sets.push([0, 6, 12, 18]);
  self.sets.push([6, 12, 18, 24]);
  self.sets.push([12, 18, 24, 30]);
  self.sets.push([18, 24, 30, 36]);

  self.sets.push([1, 7, 13, 19]);
  self.sets.push([7, 13, 19, 25]);
  self.sets.push([13, 19, 25, 31]);
  self.sets.push([19, 25, 31, 37]);

  self.sets.push([2, 8, 14, 20]);
  self.sets.push([8, 14, 20, 26]);
  self.sets.push([14, 20, 26, 32]);
  self.sets.push([20, 26, 32, 38]);

  self.sets.push([3, 9, 15, 21]);
  self.sets.push([9, 15, 21, 27]);
  self.sets.push([15, 21, 27, 33]);
  self.sets.push([21, 27, 33, 39]);

  self.sets.push([4, 10, 16, 22]);
  self.sets.push([10, 16, 22, 28]);
  self.sets.push([16, 22, 28, 34]);
  self.sets.push([22, 28, 34, 40]);

  self.sets.push([5, 11, 17, 23]);
  self.sets.push([11, 17, 23, 29]);
  self.sets.push([17, 23, 29, 35]);
  self.sets.push([23, 29, 35, 41]);

  // vertical columns
  self.sets.push([0, 1, 2, 3]);
  self.sets.push([1, 2, 3, 4]);
  self.sets.push([2, 3, 4, 5]);

  self.sets.push([6, 7, 8, 9]);
  self.sets.push([7, 8, 9, 10]);
  self.sets.push([8, 9, 10, 11]);

  self.sets.push([12, 13, 14, 15]);
  self.sets.push([13, 14, 15, 16]);
  self.sets.push([14, 15, 16, 17]);

  self.sets.push([18, 19, 20, 21]);
  self.sets.push([19, 20, 21, 22]);
  self.sets.push([20, 21, 22, 23]);

  self.sets.push([24, 25, 26, 27]);
  self.sets.push([25, 26, 27, 28]);
  self.sets.push([26, 27, 28, 29]);

  self.sets.push([30, 31, 32, 33]);
  self.sets.push([31, 32, 33, 34]);
  self.sets.push([32, 33, 34, 35]);

  self.sets.push([36, 37, 38, 39]);
  self.sets.push([37, 38, 39, 40]);
  self.sets.push([38, 39, 40, 41]);

  //diags - left to right down

  self.sets.push([2, 9, 16, 23]);

  self.sets.push([1, 8, 15, 22]);
  self.sets.push([8, 15, 22, 29]);

  self.sets.push([0, 7, 14, 21]);
  self.sets.push([7, 14, 21, 28]);
  self.sets.push([14, 21, 28, 35]);

  self.sets.push([6, 13, 20, 27]);
  self.sets.push([13, 20, 27, 34]);
  self.sets.push([20, 27, 34, 41]);

  self.sets.push([12, 19, 26, 33]);
  self.sets.push([19, 26, 33, 40]);

  self.sets.push([18, 25, 32, 39]);

  //diags - left to right up

  self.sets.push([3, 8, 13, 18]);

  self.sets.push([4, 9, 14, 19]);
  self.sets.push([9, 14, 19, 24]);

  self.sets.push([5, 10, 15, 20]);
  self.sets.push([10, 15, 20, 25]);
  self.sets.push([15, 20, 25, 30]);

  self.sets.push([11, 16, 21, 26]);
  self.sets.push([16, 21, 26, 31]);
  self.sets.push([21, 26, 31, 36]);

  self.sets.push([17, 22, 27, 32]);
  self.sets.push([22, 27, 32, 37]);

  self.sets.push([23, 28, 33, 38]);

  //self.shuffle(self.sets);

  // attach sets to squares

  // state: 0=unoc, 1=you, 2=opponent
  for (var i = 0; i < 42; i++) {
    var sqr = {
      state: 0,
      sets: []
    };

    self.squares.push(sqr);

    for (var s = 0, l = self.sets.length; s < l; s++) {

      var set = self.sets[s];

      for (var x = 0, y = set.length; x < y; x++) {
        var xy = set[x];
        if (i == xy) {
          sqr.sets.push(set);
        }
      }
    }
  }

  self.isDraw = function(){
    var dcnt = 0;
    for(var col=0;col<7;col++)
      if (!self.isOpen(col))
        dcnt++;
    return (dcnt == 7);
  };

  self.loadTops = function(){
    for(var col=0;col<7;col++)
      self.tops[col]=self.topOf(col);
    
  };

  // dropping a peice
  // 1.pick a column
  // 2.find the top of the column
  // 3.find if any sets are completed

  // are there any open spaces at this column?
  self.isOpen = function (col) {
    var tcol = col * 6;
    return self.squares[tcol].state == 0;
  }

  // get the top most space on the column
  self.topOf = function (col) {
    if (!self.isOpen(col))
      return -1;

    // return the one prior to the first one
    // found occupied
    var toc = col * 6;
    for (var i = toc + 1; i <= toc + 5; i++) {
      if (self.squares[i].state > 0)
        return (i - 1);
    }
    // whole column is open
    return toc + 5;

  }

  // do we have a winner?
  self.isWinner = function (loc, side) {
    var sets = self.squares[loc].sets;
    for (var i = 0, j = sets.length; i < j; i++) {
      var set = sets[i];
      var sidcnt = 0;
      for (var x = 0, y = set.length; x < y; x++) {
        var sqr = self.squares[set[x]];
        if (sqr.state == side)
          sidcnt++;
      }
      if (sidcnt == 4) {
        return true;
      }

    }
    return false;
  }

  // test if this spot will cause next to be winner by opponent
  self.testWinner = function(loc, side ){
    var oside = (side==1?2:1);

    var ontop = self.onTop(loc);
    if (ontop == -1) // nothing can go on top
      return false;

    // add as a fake
    self.squares[loc].state = side;

    var result = self.isWinner(ontop, oside);

    self.squares[loc].state = 0;

    return result;
  }


  self.dropIt = function (col, side) {
    var dropAt = self.topOf(col||0);

    if (dropAt > -1) {
      self.squares[dropAt].state = side;
      self.hasWinner = self.isWinner(dropAt, side);
      self.draw = self.isDraw();

      self.loadTops();
    }

  }

  self.aiDropIt = function (side) {
    var dropAt = self.aiNextSqr(side);

    if (dropAt > -1) {
      self.squares[dropAt].state = side;
      self.hasWinner = self.isWinner(dropAt, side);
      self.draw = self.isDraw();
      self.loadTops();
      return self.getCol( dropAt );
    }else{
      self.draw = true;
    }

  }

  self.colCount = function(col){
    // how many pieces in a column
    var toc = col * 6;
    var ocnt = 0;
    for (var i = toc + 1; i <= toc + 5; i++) {
      if (self.squares[i].state == 0)
        ocnt++;
    }

    return (6 - ocnt);

  }

  self.aiOpening = function(){
    var cnt = 0;
    if (self.squares[5].state>0)
      cnt++;
    if (self.squares[11].state>0)
      cnt++;
    if (self.squares[17].state>0)
      cnt++;
    if (self.squares[23].state>0)
      cnt++;
    if (self.squares[29].state>0)
      cnt++;
    if (self.squares[35].state>0)
      cnt++;
    if (self.squares[41].state>0)
      cnt++;

    return cnt <= 1;

  }

  // get the space on top of piece
  self.onTop = function(loc){
    if (loc == 0 ||loc == 6 ||loc == 12 ||loc == 18 ||loc == 24 ||loc == 30 ||loc == 36)
      return -1
    else
      return loc-1;

  }

  self.aiNextSqr = function (side) {

    var oside = (side==1?2:1);

    var smart = true;
    if (self.dumbness)
      if (self.aiShotCount % self.dumbmod == 0)
        smart = false;

    self.aiShotCount++;

    if (smart){
      // check for winner...
      // if next spot is winning move then
      // don't bother with defense..
      for(var t=0;t<7;t++){
        var ttop = self.tops[t];
        if (ttop == -1) // no more open spots
          continue;
        var tsqr = self.squares[ttop];
        for (var s=0, l=tsqr.sets.length;s<l;s++){
          var set =  tsqr.sets[s];
          // count if 3 are occupied by ai
          var acnt = 0;
          for(var i=0,j=set.length;i<j;i++){
            if( self.squares[set[i]].state == side )
              acnt++;
          }
          if(acnt == 3){
            // ai is at 3
            // next spot to take is winner
            return self.tops[t];
          }
        }
      }

      // defense - go through tops and see
      // if any will complete set for opponent
      for(var t=0;t<7;t++){
        var ttop = self.tops[t];
        if (ttop == -1) // no more open spots
          continue;
        var tsqr = self.squares[ttop];
        for (var s=0, l=tsqr.sets.length;s<l;s++){
          var set =  tsqr.sets[s];
          // count if 3 are occupied by opponent
          var ocnt = 0;

          for(var i=0;i<4;i++){
            if( self.squares[set[i]].state == oside )
              ocnt++;
          }
          if(ocnt == 3){
            // openent is at 3
            // next spot to take is top
            return self.tops[t];
          }
        }
      }

      // offense - go through tops and see
      // if any will add to ai set
      var bests=[];
      for(var t=0;t<7;t++){
        var ttop = self.tops[t];
        if (ttop == -1) // no more open spots
          continue;
        var tsqr = self.squares[ttop];
        for (var s=0, l=tsqr.sets.length;s<l;s++){
          var set =  tsqr.sets[s];
          // count if any are occupied by opponent
          // and if ai has at least one
          // add canditate sets and pick the one with
          // most ai squares
          var ocnt = acnt = 0;
          for(var i=0;i<4;i++){
            var setsqr = self.squares[set[i]];
            if( setsqr.state == oside )
              ocnt++;
            if( setsqr.state == side )
              acnt++;
          }
          if(ocnt == 0 && acnt >0){
            // openent is at 0
            // ai has at least 1

            // see if this square will sit under
            // opponent winning square...
            if (self.dumbness || (!self.testWinner(ttop, side))){

              // see if this sqr is already counting
              var fnd = false;
              for(var b=0;b<bests.length;b++){
                if (bests[b].top == self.tops[t]){
                  bests[b].cnt += acnt;
                  fnd = true;
                }
              }
              if(!fnd)
                bests.push({top:self.tops[t], cnt: acnt});

            }
          }
        }
      }

      if (bests.length){
        var best = bests[0];
        for(var b=0;b<bests.length;b++)
          if (bests[b].cnt > best.cnt)
            best = bests[b];

        return best.top;
      }

    // no defense or offense plays
    // or we are in dumbness
  } //if smart


    // check if opening move...
    if (self.aiOpening() && ! self.dumbness){
      if (self.squares[23].state==0){
          return 23;
      }else if(Math.floor(Math.random() * 2)) {
          return 17;
      }else{
          return 29;
      }

    }else{

      // get the number of open spots
      var opn = [];
      var better = [];
      for(var i=0;i<7;i++){
          var top = self.tops[i];
          // check if this would complete opponents set for opponent
          if (top>-1){
            opn.push(top);
            if (!self.dumbness){
              if (!self.testWinner(top, side)){
                better.push(top);
              }
            }
          }
        }

      if (better.length){
        // better moves
        return better[Math.floor(Math.random() * better.length)];
      }else if (opn.length){
        return opn[Math.floor(Math.random() * opn.length)];
      }
    }
    // this is not supposed to happen...
    return -1

  } // aiNextSquare


  self.getCol = function (loc) {
    return Math.floor((loc) / 6);
  }

  self.startOver = function(){
    for (var i = 0; i < 42; i++) {
      self.squares[i].state = 0;
    }
    self.hasWinner = false;
    self.draw = false;
    self.aiShotCount = 0;
    self.loadTops();
  }

  // prime tops
  self.loadTops();

}// end of board func
