'use strict'
var GOBJ = GOBJ || {};

GOBJ.level = function () {
    var self = this;

    var dbug = false;

    self.ball = null;
    self.lp1 = self.lp2 = null;

    self.create = function () {
        // make sure we are cleared out
        GOBJ.board.startOver();

        self.back_layer = self.game.add.group();
        self.mid_layer = self.game.add.group();
        self.front_layer = self.game.add.group();

        //GOBJ.audioOn = true;

        self.clack01 = self.game.add.audio('clack01');
        self.clack03 = self.game.add.audio('clack03');
        self.buttonclick01 = self.game.add.audio('buttonclick01');
        self.hallelujah = self.game.add.audio('hallelujah');
        self.hit02 = self.game.add.audio('hit02');
        self.kershluck01 = self.game.add.audio('kershluck01');
        self.whawha = self.game.add.audio('whawha');
        self.drone01 = self.game.add.audio('drone01');
        self.ding = self.game.add.audio('ding');
        self.fart = self.game.add.audio('fart');

        var hoff = self.game.world.centerX - 120;
        var voff = self.game.world.centerY; //280

        var room = self.game.add.image(0, 0, 'room');
        room.anchor.set(0.0);
        //room.width = GOBJ.window_width;
        room.height = GOBJ.window_height;

        self.back_layer.add(room);

        var gameboard = self.game.add.image(115 + hoff, voff + 80, 'gameboard');
        gameboard.anchor.set(0.5);
        gameboard.scale.set(0.85);
        self.front_layer.add(gameboard);

        self.game.physics.startSystem(Phaser.Physics.P2JS);
        self.game.physics.p2.gravity.y = 1000; //1000
        self.game.physics.p2.restitution = 0.25;

        self.walls = [];
        self.floors = [];

        self.base = self.game.add.sprite(self.game.world.centerX, 180 + voff);
        self.game.physics.p2.enable(self.base, dbug);
        self.base.body.setRectangle(self.game.width, 30);
        self.base.body.static = true;
        self.base.skey = 'base';

        for (var i = 0; i < 8; i++) {
          var wall;
          if (i == 0)
            wall = self.game.add.sprite(hoff, voff + 80);
          else if (i == 7)
            wall = self.game.add.sprite((i * 32) + hoff + 4, voff + 80);
          else
            wall = self.game.add.sprite((i * 32) + hoff + 2, voff + 65);

          self.game.physics.p2.enable(wall, dbug);
          if (i == 0 || i == 7)
            wall.body.setRectangle(10, 200);
          else
            wall.body.setRectangle(5, 160);

          wall.body.static = true;
          wall.skey = 'wall';

          self.walls.push(wall);
        }

        // floor pieces
        for (var i = 0; i < 7; i++) {
          var flr = self.game.add.sprite((i * 31.5) + hoff + 20, 150 + voff);
          self.game.physics.p2.enable(flr, dbug);
          flr.body.setRectangle(13, 20);
          flr.body.static = true;
          flr.ckey = 'floor' + i;
          flr.skey = 'floor';
          flr.findex = i;
          self.floors.push(flr);
        }

        self.pieces = [];

        self.mouseBody = self.game.add.sprite(-100, -100);
        self.game.physics.p2.enable(self.mouseBody, dbug);

        self.mouseBody.body.static = true;
        self.mouseBody.body.setCircle(10);

        self.game.input.onDown.add(function (pointer) {
          if (self.introTweenOut) {
            self.introTweenOut.start();
            self.introTweenOut = null;
          }
          if (self.pfirstTweenOut) {
            self.pfirstTweenOut.start();
            self.pfirstTweenOut = null;
          }

          if (self.yturntweenout)
            self.yturntweenout.start();

          if (Phaser.Rectangle.containsPoint(self.homebtn.getBounds(), pointer.position))
            return;
            
          if (Phaser.Rectangle.containsPoint(self.mutebtn.getBounds(), pointer.position))
            return;
          
          if (self.infobtn && Phaser.Rectangle.containsPoint(self.infobtn.getBounds(), pointer.position))
            return;

          if (self.restartbtn && Phaser.Rectangle.containsPoint(self.restartbtn.getBounds(), pointer.position))
            return;

          if (self.chatbtn && Phaser.Rectangle.containsPoint(self.chatbtn.getBounds(), pointer.position))
            return;

          // pull from anywhere
          var ppiece = self.pieces[self.pieces.length - 1];
          if (ppiece && ppiece.ai == false && ppiece.findex == -1) {
            self.mouseSpring = self.game.physics.p2.createSpring(self.mouseBody,
              ppiece.body, 100, 50, 5);
            self.htwout.start();
            if (GOBJ.audioOn)
              self.drone01.play('', 0, 1, true, false);
          }

        });

        self.game.input.addMoveCallback(function (pointer) {
          self.mouseBody.body.x = pointer.x;
          self.mouseBody.body.y = pointer.y;
        });

        self.game.input.onUp.add(function () {
          if (self.mouseSpring) {
            self.game.physics.p2.removeSpring(self.mouseSpring);
            self.mouseSpring = null;
            self.drone01.stop();
          }
        });

        // piece holder
        // move the holder in and out
        self.holdimg = self.game.add.image(-20, self.game.world.centerY - 105, 'holder');
        self.holdimg.anchor.set(0.5);
        self.holdimg.scale.set(0.85);

        self.hold1 = self.game.add.sprite(5, self.game.world.centerY - 100);
        self.front_layer.add(self.hold1);
        self.game.physics.p2.enable(self.hold1, dbug);
        self.hold1.body.setRectangle(50, 5);
        self.hold1.body.static = true;
        self.hold1.skey = '';

        //self.hold1.body.x = -100;

        self.hold2 = self.game.add.sprite(30, self.game.world.centerY - 105);
        self.front_layer.add(self.hold2);
        self.game.physics.p2.enable(self.hold2, dbug);
        self.hold2.body.setRectangle(5, 15);
        self.hold2.body.static = true;
        self.hold2.skey = '';
        //self.hold2.body.x = -100;

        self.htwin = self.game.add.tween(self.holdimg).to({
          x: 14
        }, 300, Phaser.Easing.In, false);
        self.htwin.onComplete.add(function () {
          self.hold1.body.x = 5;
          self.hold2.body.x = 30;
        });
        self.htwout = self.game.add.tween(self.holdimg).to({
          x: -20
        }, 300, Phaser.Easing.In, false);
        self.htwout.onComplete.add(function () {
          self.hold1.body.x = -100;
          self.hold2.body.x = -100;
        });

        // play again button
        self.playagainbtn = self.game.add.button(self.game.world.centerX, self.game.height * 1.25,
          'playagainbtn',
          function () {
            if (GOBJ.audioOn)
              self.buttonclick01.play('', 0, 1, false, false);
            self.playagainclear();
            if (!GOBJ.multiGame) {
              self.addPiece();
            } else {
              GOBJ.scope.askPlayAgain();
            }
          }, this, 0, 0, 1, 0);
        self.playagainbtn.anchor.setTo(0.5, 0.5);


        self.playagainclear = function () {
          self.hallelujah.stop();
          self.whawha.stop();
          self.playagaintweenout.start();
          if (self.annctweenout1)
            self.annctweenout1.start();
          if (self.annctweenout2)
            self.annctweenout2.start();
          self.clearBoard();
        }

        self.playagaintweenin = self.game.add.tween(self.playagainbtn).to({
          y: 200
        }, 500, Phaser.Easing.In, false);
        self.playagaintweenout = self.game.add.tween(self.playagainbtn).to({
          y: self.game.height * 1.25
        }, 500, Phaser.Easing.In, false);

        if (GOBJ.multiGame) {
          var itxt = '';
          if (GOBJ.youFirst)
            itxt = 'You Go First!';
          else
            itxt = 'Opponent Goes \nFirst!';

          var ptext = self.game.add.text(-(self.game.width * 1.5),
            180, itxt);
          ptext.anchor.setTo(0.5, 0.25);
          ptext.font = 'Chewy';
          ptext.fontSize = 40;
          ptext.fill = 'blue';
          ptext.align = 'center';
          ptext.setShadow(1.5, 1.5, 'red', 1);
          self.game.add.tween(ptext).to({
              x: self.game.world.centerX
            },
            500, Phaser.Easing.In, true);
          self.pfirstTweenOut = self.game.add.tween(ptext).to({
              x: -self.game.width * 1.5
            },
            500, Phaser.Easing.In, false);

          // take turns text
          var yturn = self.game.add.text(-(self.game.width * 1.5),
            50, 'It\'s your turn...');
          yturn.anchor.setTo(0.5,0);
          yturn.font = 'Chewy';
          yturn.fontSize = 40;
          yturn.fill = 'blue';
          yturn.align = 'center';
          yturn.setShadow(1.5, 1.5, 'red', 1);

          var oturn = self.game.add.text(-(self.game.width * 1.5),
            50, 'It\'s your \nopponent\'s turn...');
          oturn.anchor.setTo(0.5,0);
          oturn.font = 'Chewy';
          oturn.fontSize = 40;
          oturn.fill = 'blue';
          oturn.align = 'center';
          oturn.setShadow(1.5, 1.5, 'red', 1);

          self.yturntweenin = self.game.add.tween(yturn).to({
              x: self.game.world.centerX
            },
            500, Phaser.Easing.In, false);

          self.yturntweenout = self.game.add.tween(yturn).to({
              x: -self.game.width * 1.5
            },
            500, Phaser.Easing.In, false);

          self.oturntweenin = self.game.add.tween(oturn).to({
              x: self.game.world.centerX
            },
            500, Phaser.Easing.In, false);

          self.oturntweenout = self.game.add.tween(oturn).to({
              x: -self.game.width * 1.5
            },
            500, Phaser.Easing.In, false);


        }

        // todo: find a way to hide this text via local storage.
        // intro text
        var text = self.game.add.text(self.game.width * 1.5,
          80, 'Tap and Drag the screen\nto pick up your piece\nand release to drop it.');
        text.anchor.setTo(0.5);
        text.font = 'Chewy';
        text.fontSize = 24;
        text.fill = 'blue';
        text.align = 'center';
        //text.setShadow(2, 2, 'grey', 2);
        text.setShadow(1.5, 1.5, 'red', 1);
        self.game.add.tween(text).to({
            x: self.game.world.centerX
          },
          500, Phaser.Easing.In, true);
        self.introTweenOut = self.game.add.tween(text).to({
            x: self.game.width * 1.5
          },
          500, Phaser.Easing.In, false);

        // home button
        self.homebtn = self.game.add.button(20, self.game.height - 20, 'homebtn',
          function () {
            if (GOBJ.audioOn)
              self.buttonclick01.play('', 0, 1, false, false);

            // is user hitting back alot?
            GOBJ.backCnt++;
            if (GOBJ.backCnt > 3 && GOBJ.ads_on) {
              window.admob.showInterstitialAd();
            }

            GOBJ.scope.goHome();
            //window.location = 'index.html#/home';
          }, self, 0, 0, 0, 0);
        self.homebtn.anchor.setTo(0.5, 0.5);
        self.homebtn.scale.set(.7);

        // info button
        if(GOBJ.ads_on){  
          self.infobtn = self.game.add.button(self.game.width - 20, 20, 'infobtn',
            function () {
              if (GOBJ.audioOn)
                self.buttonclick01.play('', 0, 1, false, false);
  
              GOBJ.scope.goIAP();
  
            }, self, 0, 0, 0, 0);
          self.infobtn.anchor.setTo(0.5, 0.5);
          self.infobtn.scale.set(.7);
        }

        // mute button
        self.mutebtn = self.game.add.button(self.game.world.centerX, self.game.height - 20, 'audiobtn',
          function () {
            self.buttonclick01.play('', 0, 1, false, false);
            if (GOBJ.audioOn) {
              GOBJ.audioOn = false;
              self.mutebtn.setFrames(1, 1, 1, 1);
            } else {
              GOBJ.audioOn = true;
              self.mutebtn.setFrames(0, 0, 0, 0);
            }
          }, self, 0, 0, 0, 0);

        self.mutebtn.anchor.setTo(0.5, 0.5);
        self.mutebtn.scale.set(.7);

        if (!GOBJ.multiGame) {
          // restart button
          self.restartbtn = self.game.add.button(self.game.width - 20, self.game.height - 20, 'restartbtn',
            function () {
              if (GOBJ.audioOn)
                self.buttonclick01.play('', 0, 1, false, false);
              self.clearBoard();
              self.addPiece();

              // is user hitting back alot?
              GOBJ.backCnt++;
              if (GOBJ.backCnt > 3 && GOBJ.ads_on) {
                window.admob.showInterstitialAd();
              }

            }, self, 0, 0, 0, 0);
          self.restartbtn.anchor.setTo(0.5, 0.5);
          self.restartbtn.scale.set(-.7);
        } else {

          //chat button
          self.chatbtn = self.game.add.button(self.game.width - 20, self.game.height - 20, 'chatbtn',
            function () {
              if (GOBJ.audioOn)
                self.buttonclick01.play('', 0, 1, false, false);
              GOBJ.scope.goChat();
            }, self, 0, 0, 0, 0);
          self.chatbtn.anchor.setTo(0.5, 0.5);
          self.chatbtn.scale.set(.7);
        }

        // if multiplayer then need to find out who goes first...
        if (GOBJ.youFirst || (!GOBJ.multiGame))
          self.addPiece();

      } // create

    self.multiAgain = function () {
      GOBJ.youFirst = !GOBJ.youFirst;
      if (GOBJ.youFirst) {
        self.addPiece();
      } else {
        self.oturntweenin.start();
      }

    }

    self.chatlines = [];

    self.paintChat = function (ctxt, pno) {

      if (GOBJ.audioOn && pno != GOBJ.multiPlayerNo)
        self.ding.play('', 0, 1, false, false);

      self.chatlines.push({
        ctxt: ctxt,
        pno: pno
      });

      var tcol;
      if (pno == 1)
        tcol = 'red'
      else
        tcol = 'black'

      for (var i = 0, l = self.chatlines.length; i < l; i++) {
        var cline = self.chatlines[i];
        if (cline.gtxt) {
          self.game.add.tween(cline.gtxt).to({
              y: '-20'
            },
            500, Phaser.Easing.In, true);
        } else {
          var txt = self.game.add.text(self.game.world.centerX, self.game.height + 20, cline.ctxt);
          txt.anchor.x = 0.5;
          txt.anchor.y = 0;
          txt.font = 'Arial';
          txt.fontSize = 18;
          txt.fill = tcol;
          self.game.add.tween(txt).to({
              y: self.game.height - 70
            },
            500, Phaser.Easing.In, true);
          cline.gtxt = txt;
        }
      }

      if (self.chatlines.length > 3) {
        var rline = self.chatlines[0].gtxt;
        var fade = self.game.add.tween(rline).to({
            alpha: 0
          },
          500, Phaser.Easing.In, true);
        fade.onComplete.add(function () {
          rline.destroy();
          self.chatlines.shift();
        }, this);

      }


    }

    self.clearBoard = function () {

      for (var i = 0, l = self.pieces.length; i < l; i++) {
        var p = self.pieces[i];
        p.destroy();
      }
      self.pieces.length = 0;
      GOBJ.board.startOver();
    }

    // add a piece for this player in the holder
    self.addPiece = function () {
      self.htwin.start();
      if (GOBJ.audioOn)
        self.kershluck01.play('', 0, .25, false, false);

      if (!self.pfirstTweenOut && self.yturntweenin)
        self.yturntweenin.start();

      var pcolor = 'redp';
      if (GOBJ.multiGame)
        if (GOBJ.multiPlayerNo == 1)
          pcolor = 'redp'
        else
          pcolor = 'blackp'

      var piece = self.game.add.sprite(10, 0, pcolor);
      piece.ai = false;
      piece.scale.set(0.30);
      self.mid_layer.add(piece);
      self.game.physics.p2.enable(piece, dbug);
      piece.ckey = 'piece' + self.pieces.length;
      piece.skey = 'floor';
      piece.body.setCircle(12.75);
      piece.findex = -1;

      piece.body.onBeginContact.add(function (body, shapa, shapb, equ) {
        if (body && body.sprite.skey && GOBJ.audioOn) {
          if (body.sprite.skey == 'floor')
            self.clack01.play('', 0, 1, false, false);
          else if (body.sprite.skey == 'base')
            self.hit02.play('', 0, 1, false, false);
          else if (body.sprite.skey == 'wall')
            self.clack03.play('', 0, .5, false, true);
        }
        // body is what was hit
        if (body && body.sprite.ckey && !self.game.input.activePointer.isDown) {
          // piece is our ball or the floor

          // seems to be some abiguity here about
          // which piece is in which bucket
          var p = (equ[0].bodyA.parent.sprite.ckey == body.sprite.ckey) ? equ[0].bodyB.parent : equ[0].bodyA.parent;

          if (p.sprite.findex == -1) { // prevent setting the piece repeatidly (from bouncing)

            p.onBeginContact.removeAll();

            // the thing we hit should have the column that it's sitting in
            var col = body.sprite.findex;

            if (!GOBJ.board.isOpen(col))
              return; // no more spots at this column


            p.sprite.findex = col;

            GOBJ.board.dropIt(col, 1);

            // need a signalr call that tells opponent our move

            if (self.mouseSpring) {
              self.game.physics.p2.removeSpring(self.mouseSpring);
              self.mouseSpring = null;
            }

            if (GOBJ.multiGame)
              GOBJ.scope.movePiece(col);

            window.setTimeout(function () {
              p.static = true;
              // add a new piece
              if (GOBJ.board.hasWinner) {
                self.anncWinner(true);
                if (GOBJ.audioOn)
                  self.hallelujah.play('', 0, 1, false, false);
              } else if (GOBJ.board.draw) {
                self.anncDraw();
                if (GOBJ.audioOn)
                  self.whawha.play('', 0, 1, false, false);
              } else if (!GOBJ.multiGame) {
                self.aiPiece();
              } else {
                if (self.oturntweenin)
                  self.oturntweenin.start();
              }

            }, 1000);
          }
        }
      });

      self.pieces.push(piece);
    }

    // need a signalr event that tells us opponents move (col)

    self.aiPiece = function () {
      // get column from ai.
      self.placePiece(GOBJ.board.aiDropIt(2));
    }

    // muliplayer calls this
    self.oppPiece = function (col) {
      // get column from multiplayer.
      if (self.introTweenOut) {
        self.introTweenOut.start();
        self.introTweenOut = null;
      }
      if (self.pfirstTweenOut) {
        self.pfirstTweenOut.start();
        self.pfirstTweenOut = null;
      }

      if (self.oturntweenout)
        self.oturntweenout.start();

      GOBJ.board.dropIt(col, 2);
      self.placePiece(col);
    }

    // place oponent piece either by ai or muli
    self.placePiece = function (col) {
      // get x pos from floor pieces
      var xstart = self.floors[col].body.x;

      var pcolor = 'blackp';
      if (GOBJ.multiGame)
        if (GOBJ.multiPlayerNo == 1)
          pcolor = 'blackp'
        else
          pcolor = 'redp'


      var piece = self.game.add.sprite(xstart, 0, pcolor);
      piece.ai = true;
      self.mid_layer.add(piece);
      piece.scale.set(0.30);
      self.game.physics.p2.enable(piece, dbug);
      piece.ckey = 'piece' + self.pieces.length;
      piece.skey = 'floor';
      piece.body.setCircle(12.75);
      piece.findex = -1;

      piece.body.onBeginContact.add(function (body, shapa, shapb, equ) {

        if (body && body.sprite.skey) {
          if (body.sprite.skey == 'wall' && GOBJ.audioOn) {
            self.clack03.play('', 0, .5, false, true);
          }
        }

        // body is what was hit
        if (body && body.sprite.ckey) {
          // piece is our ball
          if (GOBJ.audioOn)
            self.clack01.play('', 0, 1, false, false);

          // seems to be some abiguity here about
          // which piece is in which bucket
          var p = (equ[0].bodyA.parent.sprite.ckey == body.sprite.ckey) ? equ[0].bodyB.parent : equ[0].bodyA.parent;

          if (p.sprite.findex == -1) {

            p.onBeginContact.removeAll();

            p.sprite.findex = body.sprite.findex;

            window.setTimeout(function () {
              p.onBeginContact.removeAll();
              p.static = true;
              // add a new piece
              if (GOBJ.board.hasWinner) {
                self.anncWinner(false);
                if (GOBJ.audioOn)
                  self.whawha.play('', 0, 1, false, false);
              } else if (GOBJ.board.draw) {
                self.anncDraw();
                if (GOBJ.audioOn)
                  self.whawha.play('', 0, 1, false, false);
              } else {
                self.addPiece();
              }

            }, 1000);

          }

        }

      });

      self.pieces.push(piece);
    }

    self.anncText = function (line1, line2) {

      var text1 = self.game.add.text(-(self.game.width * .5), 60, line1);
      text1.anchor.setTo(0.5);
      text1.font = 'Chewy';
      text1.fontSize = 40;
      text1.fill = 'blue';
      text1.setShadow(1.5, 1.5, 'red', 1);

      var text2 = self.game.add.text(self.game.width * 1.5, 120, line2);
      text2.anchor.setTo(0.5);
      text2.font = 'Chewy';
      text2.fontSize = 50;
      text2.fill = 'blue';
      text2.setShadow(1.5, 1.5, 'red', 1);
      self.game.add.tween(text1).to({
          x: self.game.world.centerX
        },
        500, Phaser.Easing.In, true);
      self.game.add.tween(text2).to({
          x: self.game.world.centerX
        },
        500, Phaser.Easing.In, true);

      self.annctweenout1 = self.game.add.tween(text1).to({
          x: -(self.game.width * .5)
        },
        500, Phaser.Easing.In, false);
      self.annctweenout2 = self.game.add.tween(text2).to({
          x: self.game.width * 1.5
        },
        500, Phaser.Easing.In, false);

    }

    self.anncWinner = function (youWon) {
      // if you lost show an ad...
      GOBJ.backCnt = 0;
      self.anncText(youWon ? 'You are the' : 'Opponent is the', 'Winner!');
      self.playagaintweenin.start();

      if (!youWon && window.admob && GOBJ.ads_on) {
        window.setTimeout(function () {
          window.admob.showInterstitialAd();
        }, 3000);

      }
    }

    self.anncDraw = function () {
      self.anncText('The game is a', 'Draw!');
      self.playagaintweenin.start();
    }

    self.oppLeft = function () {
      // opponent left the game...
      if (GOBJ.audioOn)
        self.fart.play('', 0, 1, false, false);
      self.clearBoard();
    }


  } // end of level func
