'use strict'
app.service('siglr', function ($rootScope, settings, $ionicPopup) {
  var self = this;
  $.getScript(settings.serverbase + 'signalR/hubs').done(function () {

    self.hub = $.connection.gameHub;
    //$.connection.hub.logging = true;

    // client to server

    // start room
    self.startRoom = function (gname, pkey) {
        self.hub.server.startRoom(settings.gtype, gname, pkey);
      }
      // get games waiting
    self.getGamesWaiting = function () {
      //console.log('get games waiting');
      if (self.ready) {
        self.hub.server.getGamesWaiting(settings.gtype);
      } else {
        $.connection.hub.start().done(function () {
          self.ready = true;
          self.hub.server.getGamesWaiting(settings.gtype);
        });
      }
    }

    self.joinRoom = function (gname, pkey) {

      if (self.ready) {
        self.hub.server.joinRoom(settings.gtype, gname, pkey);
      } else {
        $.connection.hub.start().done(function () {
          self.ready = true;
          self.hub.server.joinRoom(settings.gtype, gname, pkey);
        });
      }
    }

    self.leaveRoom = function (gname) {

        if (self.ready) {
          self.hub.server.leaveRoom(settings.gtype, gname);
        } else {
          $.connection.hub.start().done(function () {
            self.ready = true;
            self.hub.server.leaveRoom(settings.gtype, gname);
          });
        }
      }

    // player one calls to start game
    self.startGame = function (gname) {

        if (self.ready) {
          self.hub.server.startGame(settings.gtype, gname);
        } else {
          $.connection.hub.start().done(function () {
            self.ready = true;
            self.hub.server.startGame(settings.gtype, gname);
          });
        }
      }

    // player moves piece
      // player one calls to start game
    self.movePiece = function (gname, col) {

      if (self.ready) {
        self.hub.server.movePiece(settings.gtype, gname, col);
      } else {
        $.connection.hub.start().done(function () {
          self.ready = true;
          self.hub.server.movePiece(settings.gtype, gname, col);
        });
      }
    }

    // player chats
    self.sayChat = function (gname, txt, pno) {

        if (self.ready) {
          self.hub.server.sayChat(settings.gtype, gname, txt, pno);
        } else {
          $.connection.hub.start().done(function () {
            self.ready = true;
            self.hub.server.sayChat(settings.gtype, gname, txt, pno);
          });
        }
      }

    // ask and answer play again
    self.askPlayAgain = function (gname) {

        if (self.ready) {
          self.hub.server.askPlayAgain(settings.gtype, gname);
        } else {
          $.connection.hub.start().done(function () {
            self.ready = true;
            self.hub.server.askPlayAgain(settings.gtype, gname);
          });
        }
      }

    // respond  play again
    self.respondPlayAgain = function (gname, resp) {

      if (self.ready) {
        self.hub.server.respondPlayAgain(settings.gtype, gname, resp);
      } else {
        $.connection.hub.start().done(function () {
          self.ready = true;
          self.hub.server.respondPlayAgain(settings.gtype, gname, resp);
        });
      }

    }


    // called from the server

    self.hub.client.usrmsg = function (data) {
      $ionicPopup.alert({
        title: 'Hey...',
        template: data
      });
    }

    self.hub.client.errmsg = function (data) {
      $ionicPopup.alert({
        title: 'Whoops...',
        template: data
      });
    }

    // tell room starter that room started ok.
    self.hub.client.roomStarted = function () {
      $rootScope.$broadcast('roomStarted');
    }

    // room joined by this player
    self.hub.client.roomJoined = function (gname, pcount) {
      $rootScope.$broadcast('roomJoined', gname, pcount);
    }

    // room left by this player
    self.hub.client.roomLeft = function (gname, pcount) {
      $rootScope.$broadcast('roomLeft', gname, pcount);
    }

    // return games wating
    self.hub.client.gamesWaiting = function (data) {
      $rootScope.$broadcast('gamesWaiting', data);
    }

    // other player has left a room
    self.hub.client.playerLeft = function (gname, pcount) {
        $rootScope.$broadcast('playerLeft', gname, pcount);
      }
      // other player has joined a room
    self.hub.client.playerJoined = function (gname, has_pkey, pcount) {
      $rootScope.$broadcast('playerJoined', gname, has_pkey, pcount);
    }

    // other player has left a room
    self.hub.client.gameStarted = function (gname, firstp) {
      $rootScope.$broadcast('gameStarted', gname, firstp);
    }

    // other player moved a piece
    self.hub.client.pieceMoved = function (col) {
      $rootScope.$broadcast('pieceMoved', col);
    }

    // listen chat
    self.hub.client.listenChat = function (txt, pno) {
      $rootScope.$broadcast('listenChat', txt, pno);
    }

    // ask and respond to play again request
    self.hub.client.askedPlayAgain = function () {
      $rootScope.$broadcast('askedPlayAgain');
    }

    // ask and respond to play again request
    self.hub.client.respondedPlayAgain = function (resp) {
        $rootScope.$broadcast('respondedPlayAgain', resp);
      }
      // start the server

    $.connection.hub.url = settings.serverbase + 'signalr';

    $.connection.hub.start().done(function () {
      self.ready = true;
      $rootScope.$broadcast('ready');
      //console.log('======================================signalr connected');

    }).fail(function (error) {
      //console.log('could not open connection', error);
    });

    $.connection.hub.disconnected(function () {
      //console.log('======================================signalr disconnected');
      self.ready = false;
           setTimeout(function () {
             $.connection.hub.start().done(function () {
               self.ready = true;
               $rootScope.$broadcast('ready');
               //console.log('======================================signalr connected');

             }).fail(function (error) {
               //console.log('could not open connection', error);
             });

           }, 3000); // Restart connection after 3 seconds.

    });


  }).fail(function (error) {
    console.log("Could not load script.", error);
  });

  return self;
});
